// Copyright (c) 2022 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "AssetManager.hpp"
#include "loaders/TextureLoader.hpp"

using namespace Dawn;

void AssetManager::init() {

}

void AssetManager::update() {
  auto itPending = pendingAssetLoaders.begin();
  while(itPending != pendingAssetLoaders.end()) {
    auto loader = *itPending;
    loader->updateSync();
    loader->updateAsync();
    if(loader->loaded) {
      finishedAssetLoaders.push_back(loader);
      itPending = pendingAssetLoaders.erase(itPending);
    } else {
      itPending++;
    }
  }
}

template<>
std::shared_ptr<Texture> AssetManager::get<Texture>(
  const std::string filename
) {
  // TODO: Check for existing loader

  // Create new loader
  std::shared_ptr<TextureLoader> loader = std::make_shared<TextureLoader>(
    filename
  );
  pendingAssetLoaders.push_back(std::static_pointer_cast<AssetLoader>(loader));
  return loader->sharedTexture;
}

AssetManager::~AssetManager() {
  
}