// Copyright (c) 2022 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "dawnlibs.hpp"
#include "asset/AssetLoader.hpp"

namespace Dawn {
  class AssetManager final {
    private:
      std::vector<std::shared_ptr<AssetLoader>> pendingAssetLoaders;
      std::vector<std::shared_ptr<AssetLoader>> finishedAssetLoaders;

    public:
      /**
       * Initializes this asset manager so it can begin accepting assets.
       */
      void init();

      /**
       * Updates the asset manager.
       */
      void update();

      template<class T>
      std::shared_ptr<T> get(const std::string filename);

      /**
       * Dispose the asset manager, and all attached assets.
       */
      ~AssetManager();
  };
}