// Copyright (c) 2022 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "TextureLoader.hpp"
#include "assert/assert.hpp"

using namespace Dawn;

TextureLoader::TextureLoader(const std::string name) :
  AssetLoader(name),
  loader(name + ".texture"),
  state(TextureLoaderLoadState::INITIAL)
{
  sharedTexture = std::make_shared<Texture>();
}

void TextureLoader::updateAsync() {
  if(this->state != TextureLoaderLoadState::INITIAL) return;
  this->state = TextureLoaderLoadState::ASYNC_LOADING;
  this->loader.open();

  // Read in the header.
  uint8_t buffer[TEXTURE_LOADER_HEADER_SIZE];
  this->loader.read(buffer, TEXTURE_LOADER_HEADER_SIZE);
  size_t pos = 0;
  
  // Read Version
  pos += this->loader.readUntil(buffer, TEXTURE_LOADER_HEADER_SIZE, '|');
  std::string version = std::string((char*)buffer, pos);
  assertTrue(version == "DT_2.00", "Invalid Texture Version!");

  // Read Texture Width
  this->loader.setPosition(pos);
  pos += this->loader.readUntil(buffer, TEXTURE_LOADER_HEADER_SIZE, '|');
}

void TextureLoader::updateSync() {

}

TextureLoader::~TextureLoader() {
  this->sharedTexture = nullptr;
}